//------------------------------------------------------------------
// qfixSerial.h    Version 6
// 
// This file contains the class Serial which implements serial 
// communication (UART) for all qfix boards: 
// MiniBoard (mega32), SoccerBoard (mega128), USB-Stick (mega88)
//
// Copyright 2007-2010 by qfix robotics GmbH
// Author: Stefan Enderle
//------------------------------------------------------------------
// changes in version 6:
// - BAUD19200 added
//------------------------------------------------------------------
// changes in version 5:
// - combined version for all boards
//------------------------------------------------------------------
// changes in version 4:
// - extended for AX12 servos
//------------------------------------------------------------------


#ifndef qfixSerial_h
#define qfixSerial_h


#include "qfix.h"

static const long int CLOCK = 16000000L;		// clock 16MHz


class Serial
{
public:
  enum Baudrate { BAUD19200, BAUD38400, BAUD57600, BAUD115200, BAUD1000000 };
  
protected:
  bool outBufferEmpty();
  bool inBufferEmpty();

public:
  Serial();
  Serial(Baudrate b);
  void init(Baudrate b);
  void send(uint8_t msg);
  void send(const char* msg);
  bool bufferEmpty();
  uint8_t get();
};


Serial::Serial()
{
  init(BAUD38400);
}


Serial::Serial(Baudrate b)
{
  init(b);
}


void Serial::init(Baudrate b)
{
  long baudrate;
  
  switch (b) {
    case BAUD19200 : baudrate = 19200L;
	                  break;
    case BAUD38400 : baudrate = 38400L;
	                  break;
    case BAUD57600:  baudrate = 57600L;
	                  break;
    case BAUD115200: baudrate = 115200L;
	                  break;
    case BAUD1000000: baudrate = 1000000L;
	                  break;
    default: baudrate=0;
  }

#if defined (__AVR_ATmega128__)
  if (baudrate==1000000L) {
    UCSR1A = (1<<U2X);                // enable double speed
    UBRR1L = CLOCK / (baudrate*8L) - 1;    // speed-baud rate
  }
  else {
    UCSR1A=(0<<U2X1);                 // no double speed
    UBRR1L = CLOCK/baudrate/16L -1;   // (F_CPU / (UART_BAUD_RATE * 16L) - 1)
  }	
  UBRR1H = 0;                       // high value
  UCSR1C=(0<<UMSEL1) | (1<<UCSZ11) | (1<<UCSZ10);	// asynchronous mode, no parity, 1 stop bit, 8 bit
  UCSR1B=(1<<RXEN1)  | (1<<TXEN1);	// enable receiver and transmitter

   //  mega32  or mega8/88
#else
  if (baudrate==1000000L) {
    UCSRA = (1<<U2X);                // enable double speed
    UBRRL = CLOCK / (baudrate*8L) - 1;    // speed-baud rate
  }
  else {
    UCSRA=(0<<U2X);                  // no double speed
    UBRRL = CLOCK/baudrate/16L -1;   // (F_CPU / (UART_BAUD_RATE * 16L) - 1)
  }	
  UBRRH = 0;                       // high value
  UCSRC=(1<<URSEL) | (0<<UMSEL) | (1<<UCSZ1) | (1<<UCSZ0);	// asynchronous mode, no parity, 1 stop bit, 8 bit
  UCSRB=(1<<RXEN)  | (1<<TXEN);	// enable receiver and transmitter
#endif
}


//inline
bool Serial::outBufferEmpty()
{
#if defined (__AVR_ATmega128__)
  return UCSR1A & (1<<UDRE1);
#else   //  mega32  or mega8/88
  return UCSRA & (1<<UDRE);
#endif
}


//inline
bool Serial::inBufferEmpty()
{
#if defined (__AVR_ATmega128__)
  return !(UCSR1A & (1<<RXC1));
#else   //  mega32  or mega8/88
  return !(UCSRA & (1<<RXC));
#endif
}


//inline
bool Serial::bufferEmpty()
{
  return inBufferEmpty();
}


void Serial::send(uint8_t msg)
{
  while (!outBufferEmpty())  { }	// wait for output buffer to be empty

#if defined (__AVR_ATmega128__)
  UDR1 = msg;								// send data
#else   //  mega32  or mega8/88
  UDR = msg;								// send data
#endif

  while (!outBufferEmpty())  { }	// wait for output buffer to be empty
}


void Serial::send(const char* msg)
{
  int i=0;
  while (msg[i]!=0) {
    send(msg[i]);
	i++;
  }
}


//inline
uint8_t Serial::get()
{ 
#if defined (__AVR_ATmega128__)
  return UDR1;
#else   //  mega32  or mega8/88
  return UDR;
#endif
}


#endif


